@@ -146,7 +146,7 @@ class AnsibleInventory:
     'Parses a var and returns a string, a list or a dict'
     try:
       v_value = ast.literal_eval( raw_value )
-    except:
+    except Exception:
       v_value = raw_value
     return v_value
 
@@ -249,7 +249,7 @@ class AnsibleInventory:
     'Checks if a host has a variable with v_name with a matching value of v_value_regex'
     self.next_from_cache()
     h_vars = self.get_host_vars( host )
-    if v_name in h_vars and fullmatch( v_value_regex, h_vars[v_name] ):
+    if v_name in h_vars and fullmatch( v_value_regex, str(h_vars[v_name]) ):
       return True
     return False
 
@@ -258,7 +258,7 @@ class AnsibleInventory:
     'Checks if a group has a variable with v_name with a matching value of v_value_regex'
     self.next_from_cache()
     g_vars = self.get_group_vars( group )
-    if v_name in g_vars and fullmatch( v_value_regex, g_vars[v_name] ):
+    if v_name in g_vars and fullmatch( v_value_regex, str(g_vars[v_name]) ):
       return True
     return False
 
@@ -273,9 +273,10 @@ class AnsibleInventory:
     matching_groups = []
     for g_regex in g_regex_list:
       self.next_from_cache()
-      matching_groups += self.list_groups( g_regex )
-    if not matching_groups:
-      raise AnsibleInventory_Exception('No group matches your selection')
+      m_groups = self.list_groups( g_regex )
+      if not m_groups:
+        raise AnsibleInventory_Exception("No group matches '%s'" % g_regex)
+      matching_groups += m_groups
 
     for h_name in matching_hosts:
       for g_name in matching_groups:
@@ -339,6 +340,49 @@ class AnsibleInventory:
       if group not in self.I[g]['children']:
         self.I[g]['children'].append( group )
 
+  @write
+  def edit_host_vars(self, h_name, callback):
+    'Edits host vars by calling "callback( vars_dict )". It locks the inventory until callback returns with a new "vars_dict" with the new vars.'
+    self.next_from_cache()
+    hosts = self.list_hosts()
+    if h_name not in hosts:
+      raise AnsibleInventory_Exception('Host %s does not exist', h_name)
+
+    if h_name in self.I['_meta']['hostvars']:
+      h_vars = self.I['_meta']['hostvars'][h_name]
+    else:
+      h_vars = {}
+
+    new_vars = callback( h_vars )
+
+    if new_vars and isinstance(new_vars, dict):
+      self.I['_meta']['hostvars'][h_name] = new_vars
+
+  @write
+  def edit_group_vars(self, g_name, callback):
+    'Edits group vars by calling "callback( vars_dict )". It locks the inventory until callback returns with a new "vars_dict" with the new vars.'
+    self.next_from_cache()
+    groups = self.list_groups()
+    if g_name not in groups:
+      raise AnsibleInventory_Exception('Group %s does not exist', g_name)
+
+    if isinstance( self.I[g_name], list):
+      hosts = self.I[g_name]
+      self.I[g_name] = {
+        'hosts': hosts,
+        'vars': {},
+        'children': []
+      }
+    elif isinstance( self.I[g_name], dict) and 'vars' not in self.I[g_name]:
+      self.I[g_name]['vars'] = {}
+
+    g_vars = self.I[g_name]['vars']
+
+    new_vars = callback( g_vars )
+
+    if new_vars and isinstance(new_vars, dict):
+      self.I[g_name]['vars'] = new_vars
+
   @write
   def add_var_to_groups(self, v_name, raw_value, g_regex):
     'Adds a variable a to groups matching g_regex'
