@@ -3,14 +3,22 @@
 
 import cmd
 import json
+import os
 import readline
 import signal
 import subprocess
 import sys
+import tempfile
+import yaml
 from ansible_inventory.lib import AnsibleInventory_Color, AnsibleInventory_Exception, AnsibleInventory_WarnException, AnsibleInventory_InfoException
 from ansible_inventory.globals import VERSION, AUTHOR_NAME, AUTHOR_MAIL, URL
 
 
+class MyYamlDumper(yaml.Dumper):
+    def increase_indent(self, flow=False, indentless=False):
+        return super(MyYamlDumper, self).increase_indent(flow, False)
+
+
 class AI_Console_ValidationError(Exception):
   pass
 
@@ -174,6 +182,12 @@ class AnsibleInventory_Console(cmd.Cmd):
           ['name', 'new_value', 'in_hosts'],
           ['name', 'new_value', 'in_groups']
         ]
+      elif l2cmd == 'vars':
+        l2cmd_opts = ['in_host', 'in_group']
+        l2cmd_combs = [
+          ['in_host'],
+          ['in_group']
+        ]
 
     elif cmd == 'del':
       if l2cmd == 'host':
@@ -358,20 +372,22 @@ class AnsibleInventory_Console(cmd.Cmd):
       for v in e_vars_keys:
         if v == last_key:
           e_line+= pre_last_var
+          json_pre=' '
         else:
           e_line+= pre_middle_var
+          json_pre='│'
 
         var_line = '╴%%-%ds = %%s\n' % longest_key
 
-        if isinstance( e_vars[v], dict ):
+        if isinstance( e_vars[v], dict ) or isinstance( e_vars[v], list ):
           if self.color.use_colors:
             from pygments import highlight, lexers, formatters
             var_dict = highlight(json.dumps(e_vars[v], sort_keys=True, indent=2),
                                       lexers.JsonLexer(),
                                       formatters.Terminal256Formatter(style='vim'))
-            var_dict = var_dict.replace("\n", "\n  │       │ ")
+            var_dict = var_dict.replace("\n", "\n  │       %s " % json_pre)
           else:
-            var_dict = json.dumps(e_vars[v], sort_keys=True, indent=2).replace("\n", "\n  │       │ ")
+            var_dict = json.dumps(e_vars[v], sort_keys=True, indent=2).replace("\n", "\n  │       %s " % json_pre)
           e_line+= var_line % (self.C(v), var_dict)
         else:
           e_line+= var_line % (self.C(v), e_vars[v])
@@ -383,7 +399,7 @@ class AnsibleInventory_Console(cmd.Cmd):
       rows, columns = subprocess.check_output(['stty', 'size']).split()
       rows = int(rows)
       columns = int(columns)
-    except:
+    except Exception:
       rows, columns = 25, 80
     lines=start
     line_cols=start.__len__()
@@ -758,6 +774,47 @@ class AnsibleInventory_Console(cmd.Cmd):
           self.inventory.change_host_var(v_name, new_value, h_regex)
         self.__ok('Variable %s changed to %s in selected hosts' % (self.C(v_name), self.C(new_value)))
 
+  def __editor( self, dict_data ):
+    "Converts 'dict_data to YAML and places it in a file, which is opened with $EDITOR and the result is converted back to dict and returned'"
+    tmpfile = tempfile.mkstemp( suffix=".yml", text=True )
+    new_dict_data = None
+    editor = os.getenv('EDITOR')
+    if not editor:
+      self.__error( 'The EDITOR environment variable is empty.' )
+
+    try:
+      with open(tmpfile[1], 'w') as t:
+        yaml.dump( dict_data, t, Dumper=MyYamlDumper, default_flow_style=False, indent=4)
+    except Exception:
+      self.__error( 'Could not load current variables')
+      return dict_data
+
+    try:
+      os.system( "%s %s" % (editor, tmpfile[1]) )
+    except Exception:
+      self.__error( 'Error running the editor')
+      return dict_data
+
+    try:
+      with open(tmpfile[1]) as t:
+        new_dict_data = yaml.full_load(t)
+    except Exception:
+      self.__error( 'Could not write the new variables')
+      return dict_data
+
+    if not new_dict_data:
+      return dict_data
+
+    return new_dict_data
+
+  def __edit_vars( self, args_opt ):
+    'Handle "edit vars" which opens $EDITOR'
+
+    if 'in_group' in args_opt:
+      self.inventory.edit_group_vars( args_opt['in_group'], self.__editor )
+    if 'in_host' in args_opt:
+      self.inventory.edit_host_vars( args_opt['in_host'], self.__editor )
+
   @console_handler
   def do_edit(self, args):
     """
@@ -766,12 +823,16 @@ class AnsibleInventory_Console(cmd.Cmd):
     edit group <[name=]GROUP> <new_name=NEW_NAME>
     edit var   <[name=]VAR_NAME> <new_name=NEW_NAME> <[in_hosts=HOST_REGEX_LIST | in_groups=GROUP_REGEX_LIST]>
     edit var   <[name=]VAR_NAME> <new_value=NEW_VALUE> <[in_hosts=HOST_REGEX_LIST | in_groups=GROUP_REGEX_LIST]>
+    edit vars  <[in_host=HOST | in_group=GROUP]>
 
     HOST: Domain name or IP of the host
+    GROUP: Group name
     VAR_NAME: Variable name
     VAR_VALUE: Variable value
     {SOMETHING}_REGEX: Regular expression (i.e.: name=test_.* )
     {SOMETHING}_REGEX_LIST: Comma separated list of regular expressions (i.e.: in_groups=test[1-3],example.*)
+
+    NOTE: "edit vars" will open your $EDITOR to edit the vars in YAML format.
     """
     try:
       args = self.__validate_args( args )
@@ -795,6 +856,10 @@ class AnsibleInventory_Console(cmd.Cmd):
     if args_pos[1] == 'var':
       self.__edit_var( args_opt )
 
+    # EDIT VARS
+    if args_pos[1] == 'vars':
+      self.__edit_vars( args_opt )
+
   def __del_host( self, h_regex, from_groups ):
     'Handle "del host"'
 
@@ -942,113 +1007,165 @@ class AnsibleInventory_Console(cmd.Cmd):
     if args_pos[1] == 'var':
       self.__del_var( name, from_groups, from_hosts )
 
+  def completenames(self, text, *ignored):
+    """Override class method to add an space after main command completion"""
+    return [a+' ' for a in super(AnsibleInventory_Console, self).completenames(text, *ignored)]
+
   def completedefault(self, text, line, begidx, endidx):
     current_line=line.split()
-    cmd = current_line[ 0 ]
+    wildcards = ('HOSTS', 'VARS', 'GROUPS')  # Used to autocomplete hosts, vars and groups
     cmd_map = {
       'add' : {
-        'host' : { 'host': None, 'to_groups': None },
-        'group': { 'to_groups': None },
-        'var'  : { 'to_hosts': None, 'to_groups': None }
-      },
-      'del' : {
-        'host' : { 'from_groups': None },
-        'group': { 'from_groups': None },
-        'var'  : { 'from_hosts': None, 'from_groups': None }
-      },
-      'edit': {
-        'host' : { 'new_name': None, 'new_host': None },
-        'group': { 'new_name': None },
+        'host' : {
+          'HOSTS': {
+            'host=': None,
+            'to_groups=': 'GROUPS'
+          }
+        },
+        'group': {
+          'GROUPS': {
+            'to_groups=': 'GROUPS'
+          }
+        },
         'var'  : {
-          'new_name' : { 'in_hosts': None, 'in_groups': None },
-          'new_value': { 'in_hosts': None, 'in_groups': None },
+          'VARS': {
+            'to_hosts=': 'HOSTS',
+            'to_groups=': 'GROUPS'
+          }
         }
       },
+      'del' : {
+        'host' : {
+          'HOSTS': {
+            'from_groups=': 'GROUPS'
+          }
+        },
+        'group': {
+          'GROUPS': {
+            'from_groups=': 'GROUPS'
+          }
+        },
+        'var'  : {
+          'VARS': {
+            'from_hosts=': 'HOSTS',
+            'from_groups=': 'GROUPS'
+          }
+        },
+      },
+      'edit': {
+        'host' : {
+          'HOSTS': {'new_name=': None, 'new_host=': None },
+        },
+        'group': {
+          'GROUPS': { 'new_name=': None },
+        },
+        'var'  : {
+          'VARS': {
+            'new_name' : { 'in_hosts=': 'HOSTS', 'in_groups=': 'GROUPS' },
+            'new_value': { 'in_hosts=': 'HOSTS', 'in_groups=': 'GROUPS' },
+          }
+        },
+        'vars' : { 'in_host=': 'HOSTS', 'in_group=': 'GROUPS'},
+      },
       'show': {
-        'host': { 'in_groups': None },
-        'hosts': { 'in_groups': None },
-        'group': {},
-        'groups': {},
+        'host': {
+          'HOSTS': {
+            'in_groups=': 'GROUPS',
+            'VARS': { 'VARS': { 'VARS': None } }
+          }
+        },
+        'hosts': {
+          'HOSTS': {
+            'in_groups=': 'GROUPS',
+            'VARS': { 'VARS': { 'VARS': None } }
+          }
+        },
+        'group': {
+          'GROUPS': {
+            'VARS': { 'VARS': { 'VARS': None } }
+          }
+        },
+        'groups': {
+          'GROUPS': {
+            'VARS': { 'VARS': { 'VARS': None } }
+          }
+        },
         'tree': {}
       },
     }
 
     def __comp( _kind, _text ):
       self.inventory.next_from_cache()
-      if _kind == 'host':
+      if _kind in ('host', 'HOSTS'):
         return self.inventory.list_hosts( _text+'.*' )
-      elif _kind == 'var':
+      elif _kind in ('var', 'VARS'):
         return self.inventory.list_vars( _text+'.*' )
       else:
         return self.inventory.list_groups( _text+'.*' )
 
-    if cmd in cmd_map:
-      if current_line.__len__() > 1:
-        kind = current_line[ 1 ]
-        if kind not in cmd_map[ cmd ].keys():
-          for sc in cmd_map[ cmd ]:
-            if sc.startswith( kind ):
-              return [ sc+' ' ]
+    def __get_completions( _list, _text, _prepend='' ):
+      possibles = []
+      for i in _list:
+        # autocomplete hosts, vars or groups
+        if i in wildcards:
+          post=''
+          if i == 'VARS':
+            post='='
+          if _prepend:
+            _text = _text[len( _prepend ):]
+          for p in __comp(i, _text):
+            possibles.append( _prepend+p+post )
+        elif i.startswith( _text ):
+          possibles.append( i )
+      return possibles
+
+    def __completion_matches( _key, _list ):
+      count=0
+      for k in list(_list):
+        if k.startswith( _key ):
+          count+=1
+      return count
+
+    if line[-1] == ' ':
+      # We need to check for the next command
+      current_partial = ''
+    else:
+      current_partial = current_line[-1]
+
+    current_dict_cmd = cmd_map
+    prepend=''
+    for pc in current_line:
+      wildcards_in_keys = list(set(current_dict_cmd.keys()) & set(wildcards))
+      # Not completing, just geting the path to follow
+      if wildcards_in_keys and not pc == current_partial:
+        #  ^-- On a wildcard           ^-- Not completing the wildcard anymore
+        current_dict_cmd = current_dict_cmd[ wildcards_in_keys[0] ]
+        prepend=''
+      elif pc.split('=')[0]+"=" in current_dict_cmd.keys():
+        current_dict_cmd = {current_dict_cmd[ pc.split('=')[0]+"=" ]: None}
+        prepend=pc.split('=')[0]+"="
+        if ',' in pc:
+          prepend+=','.join(pc.split('=')[1].split(',')[:-1])+','
+      elif pc in current_dict_cmd.keys():
+        if pc == current_partial:
+          # if we have just completed, return to add trailing space
+          break
+        if __completion_matches( pc, current_dict_cmd.keys() ) == 1 or not current_partial.startswith( pc ):
+          current_dict_cmd = current_dict_cmd[ pc ]
+          prepend=''
+
+    completions = __get_completions( current_dict_cmd.keys(), current_partial, prepend)
+
+    if current_partial:
+      if not completions:
+        return [ current_partial ]
+
+      if completions.__len__() == 1:
+        if completions[0][-1] in ('=', ','):
+          return [ completions[0] ]
+        elif prepend:
+          return [ completions[0] ]
         else:
-          options = list(cmd_map[ cmd ][ kind ].keys())
+          return [ completions[0] + ' ' ]
 
-          if current_line.__len__() > 2:
-            target = current_line[ 2 ]
-          else:
-            target = None
-
-          if current_line.__len__() > 3:
-            subtarget = current_line[ 3 ]
-            options = list(cmd_map[ cmd ][ kind ].keys())
-          else:
-            subtarget = None
-
-          if current_line.__len__() > 4 or (current_line.__len__() == 4 and not text):
-            if cmd == 'edit' and kind == 'var' and subtarget.startswith('new_'):
-              options = list(cmd_map[ cmd ][ kind ][ subtarget.split('=')[0] ].keys())
-              if current_line.__len__() > 4:
-                subtarget = current_line[ 4 ]
-              else:
-                subtarget = None
-
-          if cmd == 'show':
-            if kind in ('host', 'hosts'):
-              if target is None or target in 'in_groups':
-                return __comp( 'host', text ) + ['in_groups=']
-              elif target.startswith('in_groups='):
-                if subtarget is None and text != '':
-                  subtarget = target
-                else:
-                  return __comp( 'var', text )
-            if kind in ('group', 'groups'):
-              if target and not text:
-                return __comp( 'var', text )
-              else:
-                return __comp( 'group', text )
-
-          if subtarget is not None:
-            if 'groups=' in text or 'hosts=' in text:
-              part = text.split('=')[1].split(',')[ -1 ]
-              prepart = text.rpartition(',')[0]
-              if prepart:
-                prepart+=','
-              else:
-                prepart = text.split('=')[0]+'='
-              if 'hosts=' in text:
-                return [ prepart+x for x in __comp('host', part) ]
-              return [ prepart+x for x in __comp('group', part) ]
-            else:
-              opt = []
-              for o in [ op+'=' for op in options ]:
-                if o.startswith( text ):
-                  opt.append( o )
-              return opt or options
-
-          if target is not None:
-            if text:
-              return __comp( kind, text )
-            return options
-          return __comp( kind, text )
-      else:
-        return list(cmd_map[ cmd ].keys())
-    return []
+    return completions
